@@ -2,51 +2,18 @@
 
 u32 seconds;
 
-/* create the map */
+extern u32 gba_bank;
 
-#define MAP_SIZE_X 32
-#define MAP_SIZE_Y 32
+extern const unsigned char map_Bitmap[1024];
 
-/*u8 map[] = {
-	1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
-	1, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1,
-	1, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1,
-	1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 0, 1,
-	1, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 1,
-	1, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 1,
-	1, 0, 0, 0, 1, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 1,
-	1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1};
-*/
+const u8* map = map_Bitmap;
 
-#include "map.raw.c"
+extern const unsigned char walltextures_Bitmap[16384];
+extern const unsigned short walltextures_Palette[256];
 
-s32 px = 400;
-s32 py = 160;
-s32 pa = 180;
-
-#define SCREEN_WIDTH 120
-#define SCREEN_HEIGHT 160
-
-//u8 scanline_buffer[2 * SCREEN_HEIGHT];
-
-#define PROJECTION_DISTANCE 208
-//#define PROJECTION_DISTANCE 104
-
-int pythag(float dx, float dy)
-{
-	return sqrt(dx*dx + dy*dy);
-}
-
-/*double gettimeofday_double()
-{
-	struct timeval tv;
-	double foo;
-	gettimeofday(&tv, NULL);
-	
-	foo = tv.tv_usec; foo/= 1000000.0;
-	foo+= tv.tv_sec;
-	return foo;
-}*/
+//s32 px = 400; s32 py = 160; s32 pa = 180;
+s32 px = 350; s32 py = 700; s32 pa = 270;
+//s32 px = 264; s32 py = 771; s32 pa = 355;
 
 #define MOVE_UP		1 << 0
 #define MOVE_DOWN	1 << 1
@@ -55,8 +22,6 @@ int pythag(float dx, float dy)
 #define MOVE_TL		1 << 4
 #define MOVE_TR		1 << 5
 
-#include "walltextures.raw.c"
-#include "master.pal.c"
 
 //#define TRACE(x...) printf(x)
 //#define TRACE(x...) sprintf(foo, x); print(foo)
@@ -76,22 +41,9 @@ void print(char *s)
 		
 }
 
-void asm_rasterise_line(u32 x, s32 slice_height, const u8* tex, u32 texture_offset) CODE_IN_IWRAM;
-
-#define USE_ASM_RASTER
+void asm_rasterise_line(u32 x, s32 slice_height, const u8* tex, u32 texture_offset, u32 l, u32 h, u8* paltable) CODE_IN_IWRAM;
 
 #ifdef USE_ASM_RASTER
-/*void rasterise_line(u32 x, s32 slice_height, u32 texture_offset, u8* tex)
-{
-	int tex_inc_high, tex_inc_low;
-	
-	tex_inc_high = 64 / slice_height;
-	tex_inc_low = (((64 % slice_height) << 16) / slice_height);
-
-	asm_rasterise_line(x, slice_height, tex, texture_offset, tex_inc_low, tex_inc_high);
-}*/
-
-#else
 void rasterise_line(u32 x, s32 slice_height, u32 texture_offset)
 {
 	int y;
@@ -113,7 +65,6 @@ void rasterise_line(u32 x, s32 slice_height, u32 texture_offset)
 
 	if(slice_height > SCREEN_HEIGHT)
 	{
-		//tex_pos = (-wall_start_y) * 64 / slice_height;
 		tex_low = (-wall_start_y) * tex_inc_low;
 		tex_pos = (-wall_start_y) * tex_inc_high + (tex_low >> 16);
 		tex_low &= 0x0000ffff;
@@ -123,7 +74,6 @@ void rasterise_line(u32 x, s32 slice_height, u32 texture_offset)
 
 	for(y = wall_start_y; y < wall_end_y ; y++)
 	{
-		//int y_pos = y + ((SCREEN_HEIGHT - slice_height) / 2);
 		if(y >= 0 && y < SCREEN_HEIGHT)
 		{
 			u16 pix = wall_Bitmap[(tex_pos * 64) + texture_offset];
@@ -146,12 +96,57 @@ void rasterise_line(u32 x, s32 slice_height, u32 texture_offset)
 	}
 }
 #endif
+
+u16 sliceheights[SLICEHEIGHT_TABLE_SIZE];
+
+u8 texinc_high_table[TEXINC_TABLE_SIZE];
+u16 texinc_low_table[TEXINC_TABLE_SIZE];
+
+u16 floor_dist_table[SCREEN_HEIGHT];
+
+u8 paltable[256 * 8];
+
+void build_sliceheight_table()
+{
+	int i;
+	for(i = 0; i < SLICEHEIGHT_TABLE_SIZE; i++)
+	{
+		sliceheights[i] = (64 * PROJECTION_DISTANCE)  / i;
+	}
+	for(i = 0; i < TEXINC_TABLE_SIZE; i++)
+	{
+		texinc_high_table[i] = 64 / i;
+		texinc_low_table[i] = (((64 % i) << 16) / i);
+	}
+	for(i=SCREEN_HEIGHT/2; i < SCREEN_HEIGHT; i++)
+	{
+		floor_dist_table[i-(SCREEN_HEIGHT/2)] = (64/*(SCREEN_HEIGHT/2)*/ * PROJECTION_DISTANCE) / (2*(i - (SCREEN_HEIGHT/2)));
+	}
+
+	for(i=0; i < 8; i++)
+	{
+		int c;
+		for(c=0; c < 256; c++)
+		{
+			if(i*32 + c < 256)
+			{
+				paltable[i*256 + c] = i*32 + c;
+			} else {
+				paltable[i*256 + c] = 0;
+			}
+		}
+	}
+}
+void asm_floor_draw(u32 x, s32 slice_height, const u8* tex, s32 cos_a, s32 sin_a, s32 one_over_cos_b, s32 px, s32 py) CODE_IN_IWRAM;
+
 void raycast()
 {
 	int i;
 	char foo[100];
 	s32 dbuf[SCREEN_WIDTH];
-	
+	int stime, etime;
+
+	stime = seconds;
 	/* scan across the screen */
 	for(i=0; i < SCREEN_WIDTH; i++)
 	{
@@ -163,12 +158,12 @@ void raycast()
 		int wall_hit_x, wall_hit_y;
 		int distance;
 		int d1, d2;
-
+		int tex_inc_low, tex_inc_high;
 		int slice_height;
 		int texture_offset; 
 		
 		if(angle < FIXED(0)) angle += FIXED(360);
-		if(angle > FIXED(360)) angle -= FIXED(360);
+		if(angle >= FIXED(360)) angle -= FIXED(360);
 		
 		TRACE("checking ray %d, angle %f\n", i, angle / 256.0);
 		
@@ -191,14 +186,18 @@ void raycast()
 		}
 		
 		/* now get horiz co-ord of first grid intersection */
-		if(gba_tan(angle))
+		if(angle == FIXED(90) || angle == FIXED(270))
 		{
-			ax = FIXED(px) + ((py - ay) << 16) / gba_tan(angle);
+			ax = FIXED(px); Xa = 0;
+		} else if(angle == FIXED(0) || angle == FIXED(180)) {
+			ax = 1 << 24; Xa = 1 << 24;
+		} else {
+			//ax = FIXED(px) + ((py - ay) << 16) / gba_tan(angle);
+			ax = FIXED(px) + ((py - ay)) * gba_one_over_tan(angle);
 
 			/* calculate X increment between intersections */
-			Xa = (-Ya << 8) / gba_tan(angle);
-		} else {
-			ax = 1 << 24; Xa = 1 << 24;
+			//Xa = (-Ya << 8) / gba_tan(angle);
+			Xa = ((-Ya) * gba_one_over_tan(angle)) >> 8;
 		}
 		
 		/* loop until we hit a wall */
@@ -227,13 +226,15 @@ void raycast()
 		TRACE("wall found!\n");
 		cur_x = cur_x >> 8; cur_y = cur_y >> 8;
 		wall_hit_x = cur_x; wall_hit_y = cur_y;
-		if(abs(px - cur_x) > abs(py - cur_y))
+		/*if(abs(px - cur_x) > abs(py - cur_y))
 		{
-			d1 = abs(((px - cur_x) << 8) / gba_cos(angle));
+			//d1 = abs(((px - cur_x) << 8) / gba_cos(angle));
+			d1 = abs(((px - cur_x) * gba_one_over_cos(angle)) >> 8);
 		} else {
-			d1 = abs(((py - cur_y) << 8) / gba_sin(angle));
-		}
-		//d1 = pythag(px - cur_x, py - cur_y);
+			//d1 = abs(((py - cur_y) << 8) / gba_sin(angle));
+			d1 = abs(((py - cur_y) * gba_one_over_sin(angle)) >> 8);
+		}*/
+		d1 = pythag(px - cur_x, py - cur_y);
 		TRACE("distance: %d\n", d1);
 		
 		/* trace vertically */
@@ -254,16 +255,22 @@ void raycast()
 		}
 		
 		/* now get horiz co-ord of first grid intersection */
-		ay = FIXED(py) + (((px - ax)*gba_tan(angle)));
-
-		/* calculate X increment between intersections */
-		Ya = ((-Xa) * gba_tan(angle)) >> 8;
+		if(angle == FIXED(90) || angle == FIXED(270))
+		{
+			ay = 1 << 24; Ya = 1 << 24;
+		} else if(angle == FIXED(0) || angle == FIXED(180)) {
+			ay = FIXED(py); Ya = 0;
+		} else {
+			ay = FIXED(py) + (((px - ax)*gba_tan(angle)));
 
+			/* calculate X increment between intersections */
+			Ya = ((-Xa) * gba_tan(angle)) >> 8;
+		}
 		/* loop until we hit a wall */
 		cur_x = FIXED(ax); cur_y = ay;
 
 		TRACE("Tracing vertically...\n");
-		TRACE("Starting at %d/%d, inc by %d/%d\n", ax, ay, Xa, Ya);
+		TRACE("Starting at %d/%d, inc by %d/%d\n", cur_x, ay, Xa, Ya);
 		
 		while(1)
 		{
@@ -287,22 +294,18 @@ void raycast()
 		/* check which of the horizontal trace or the vertical trace found the
 		 * nearer wall */
 		cur_x = cur_x >> 8; cur_y = cur_y >> 8;
-		if(abs(px - cur_x) > abs(py - cur_y))
-		{
-			d2 = abs(((px - cur_x) << 8) / gba_cos(angle));
-		} else {
-			d2 = abs(((py - cur_y) << 8)/ gba_sin(angle));
-		}
-		//d2 = pythag(px - cur_x, py - cur_y);
+		d2 = pythag(px - cur_x, py - cur_y);
 		TRACE("distance: %d\n", d2);
 		
 		if(d2 < d1)
 		{
 			wall_hit_x = cur_x; wall_hit_y = cur_y;
 			texture_offset = wall_hit_y % 64;
+			//texture_offset = wall_hit_y - ((wall_hit_y >> 6) << 6);
 			distance = d2;
 		} else {
 			texture_offset = wall_hit_x % 64;
+			//texture_offset = wall_hit_x - ((wall_hit_x >> 6) << 6);
 			distance = d1;
 		}
 		TRACE("Nearest wall hit at %d/%d\n", wall_hit_x, wall_hit_y);
@@ -315,15 +318,46 @@ void raycast()
 
 		TRACE("adjusted distance: %d\n", distance);
 
-		slice_height = FIXED(64 * PROJECTION_DISTANCE)  / distance;
-		dbuf[i] = FIXED_TO_INT(distance);
+		if(distance < FIXED(SLICEHEIGHT_TABLE_SIZE))
+		{
+			slice_height = sliceheights[FIXED_TO_INT(distance)];
+		} else {
+			slice_height = FIXED(64 * PROJECTION_DISTANCE)  / distance;
+		}
 
+		if(slice_height < TEXINC_TABLE_SIZE)
+		{
+			tex_inc_low = texinc_low_table[slice_height];
+			tex_inc_high = texinc_high_table[slice_height];
+		} else {
+			tex_inc_high = 64 / slice_height;
+			tex_inc_low = (((64 % slice_height) << 16) / slice_height);
+		}
+		
+		dbuf[i] = FIXED_TO_INT(distance);
+		TRACE("slice height: %d\n", slice_height);
 		/* draw here! */
 		asm_rasterise_line(i, slice_height, walltextures_Bitmap + 
 			(map[((wall_hit_y/64) * MAP_SIZE_X) + (wall_hit_x/64)] - 1)*64*64,
-			texture_offset);
+			texture_offset, tex_inc_low, tex_inc_high,
+			paltable + ((FIXED_TO_INT(distance)/128) *256));
+		//sprintf(foo, "%d\n", 	(FIXED_TO_INT(distance)/200) * 32); print(foo);
+	
+		/* floor textures! */
+		if(slice_height < SCREEN_HEIGHT)
+		{
+			asm_floor_draw(i, slice_height, walltextures_Bitmap, gba_cos(angle),
+				gba_sin(angle), gba_one_over_cos(abs(FIXED(pa) - angle)), 
+				px, py); 
+		}
 	}
+	etime = seconds;
+	//sprintf(foo, "raycast time: %d\n", etime - stime); print(foo);
+	
+	stime = seconds;
 	objs_draw(px, py, pa, dbuf);
+	etime = seconds;
+	//sprintf(foo, "objdraw time: %d\n", etime - stime); print(foo);
 }
 
 void player_move(s32 angle)
@@ -343,23 +377,57 @@ void player_move(s32 angle)
 		py = py - (((speed) * gba_sin(angle)) >> 8);
 }
 
+//s32 asmgba_sin(s32 angle) CODE_IN_IWRAM;
+
+#if 0
 void trigtest()
 {
-	int a;
+	int a, i;
+	int stime, etime;
+	
 	char foo[100];
+	
 	for(a=0; a < 360; a+=10)
 	{
 		sprintf(foo, "sin(%3d): %f %f\n", a, sin(a * (M_PI / 180.0)), gba_sin(a*256) / 256.0); print(foo);
 	}
 	for(a=0; a < 360; a+=10)
 	{
-		sprintf(foo, "cos(%3d): %f %f\n", a, cos(a * (M_PI / 180.0)), gba_cos(a*256) / 256.0); print(foo);
+		//sprintf(foo, "cos(%3d): %f %f\n", a, cos(a * (M_PI / 180.0)), gba_cos(a*256) / 256.0); print(foo);
 	}
 	for(a=0; a < 360; a+=10)
 	{
-		sprintf(foo, "tan(%3d): %f %f\n", a, tan(a * (M_PI / 180.0)), gba_tan(a*256) / 256.0); print(foo);
+		//sprintf(foo, "tan(%3d): %f %f\n", a, tan(a * (M_PI / 180.0)), gba_tan(a*256) / 256.0); print(foo);
 	}
+
+	// trigbench!
+#if 0
+	stime = seconds;
+	for(i=0; i < 10000; i++)
+	{
+		for(a=0; a < 360; a++)
+		{
+			gba_sin(a << 8);
+		}
+	}
+	etime = seconds;
+	sprintf(foo, "C sin: %d\n", etime - stime); print(foo);
+
+	stime = seconds;
+	for(i=0; i < 10000; i++)
+	{
+		for(a=0; a < 360; a++)
+		{
+			asmgba_sin(a << 8);
+		}
+	}
+	etime = seconds;
+	sprintf(foo, "asm sin: %d\n", etime - stime); print(foo);
+#endif
 }
+#endif
+
+void asm_fpsmeter(u32 fps) CODE_IN_IWRAM;
 
 int AgbMain()
 {
@@ -369,9 +437,12 @@ int AgbMain()
 	u32 curtime, lasttime;
 	int framewait = 0;
 	u32 movement = 0;
+	int fps = 0;
+
+	build_sliceheight_table();
 	
 	REG_TM2CNT = TIMER_FREQ_64 | TIMER_IRQ_ENABLE | TIMER_ENABLE;
-	REG_TM2D = (65536 - 262);
+	REG_TM2D = (65536 - 26);
 	interrupt_init();
 	REG_IE |= INT_TM2; // enable timer 2 for seconds counter
   REG_IME = 1;
@@ -379,7 +450,7 @@ int AgbMain()
 	//trigtest();
 	SetMode(MODE_4 | BG2_ENABLE);
 	for(i=0; i < 256; i++)
-		BGPaletteMem[i] = master_Palette[i];
+		BGPaletteMem[i] = walltextures_Palette[i];
 	
 	gba_initbank();
 
@@ -409,7 +480,7 @@ int AgbMain()
 			pa += 5;
 		}
 		if(pa < 0) pa += 360;
-		if(pa > 360) pa -= 360;
+		if(pa >= 360) pa -= 360;
 
 		//sprintf(foo, "px: %4d py: %4d pa: %3d\n", px, py, pa); print(foo);
 
@@ -440,18 +511,20 @@ int AgbMain()
 		}
 		
 		raycast();
-		gba_vsync();
+		asm_fpsmeter(fps);
+
+		//gba_vsync();
 		gba_swapbank();
 		framewait++;
 		
 		if(framewait == 10)
 		{
 			curtime = seconds;
-			//sprintf(foo, "FPS: %d\n", (10 * 1000) / (curtime - lasttime)); print(foo);
+			fps = 20 * 1000 / (curtime - lasttime);
+			//sprintf(foo, "FPS: %f\n", (10.0 * 1000) / (curtime - lasttime)); print(foo);
 			lasttime = curtime;
 			framewait = 0;
 		}
-		//usleep(100000);
 	}
 
 	return 0;
